﻿using System;
using System.Collections.Generic;
using System.DirectoryServices;
using System.DirectoryServices.AccountManagement;
using System.Linq;

namespace Crossrail.IMS.Common
{
    public class ActiveDirectoryUser
    {
        public string UserName { get; set; }
        public string DisplayName { get; set; }
        public string EmailAddress { get; set; }
        public string ContactNumber { get; set; }
    }

    [DirectoryRdnPrefix("CN")]
    [DirectoryObjectClass("person")]
    public class UserPrincipalEx: UserPrincipal
    {
        private const string TELEPHONE_NUMBER = "telephoneNumber";
        private const string HOME_PHONE = "homePhone";
        private const string OTHER_HOME_PHONE = "otherHomePhone";
        private const string MOBILE = "mobile";
        private const string OTHER_MOBILE = "otherMobile";

        public UserPrincipalEx(PrincipalContext context)
            :base(context)
        {
            
        }

        public UserPrincipalEx(PrincipalContext context, string samAccountName, string password, bool enabled)
            : base(context, samAccountName, password, enabled)
        {

        }

        private string GetExProperty(string propertyName)
        {
            var result = ExtensionGet(propertyName);
            if (result.Length != 1)
                return string.Empty;

            return (string) result[0];
        }
        
        public string GetProperty(string propertyName)
        {
            var directoryEntry = GetUnderlyingObject() as DirectoryEntry;
            if (directoryEntry == null)
                return string.Empty;

            return (string)directoryEntry.Properties[propertyName].Value;
        }

        [DirectoryProperty(TELEPHONE_NUMBER)]
        public string TelephoneNumber
        {
            get { return GetExProperty(TELEPHONE_NUMBER); }
            set { ExtensionSet(TELEPHONE_NUMBER, value); }
        }
 
        [DirectoryProperty(HOME_PHONE)]
        public string HomePhone
        {
            get { return GetExProperty(HOME_PHONE); }
            set { ExtensionSet(HOME_PHONE, value); }
        }

        [DirectoryProperty(OTHER_HOME_PHONE)]
        public string OtherHomePhone
        {
            get { return GetExProperty(OTHER_HOME_PHONE);}
            set { ExtensionSet(OTHER_HOME_PHONE, value); }
        }

        [DirectoryProperty(MOBILE)]
        public string Mobile
        {
            get { return GetExProperty(MOBILE); }
            set { ExtensionSet(MOBILE, value); }
        }

        [DirectoryProperty(OTHER_MOBILE)]
        public string OtherMobile
        {
            get { return GetExProperty(OTHER_MOBILE); }
            set { ExtensionSet(OTHER_MOBILE, value); }
        }
    }

    public class ActiveDirectoryHelper
    {
        private static string Domain
        {
            get { return System.Configuration.ConfigurationManager.AppSettings["Domain"]; }
        }

        public static List<ActiveDirectoryUser> SearchUser(string userName)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var user = new UserPrincipalEx(context))
                {
                    user.SamAccountName = userName + "*";
                    user.Enabled = true;
                    using (var pS = new PrincipalSearcher())
                    {
                        pS.QueryFilter = user;
                        return pS.FindAll().Select(r => new ActiveDirectoryUser()
                        {
                            UserName = r.SamAccountName,
                            DisplayName = r.DisplayName,
                            EmailAddress = ((r as UserPrincipalEx) == null) ? string.Empty : ((UserPrincipalEx)r).EmailAddress,
                            ContactNumber = ((r as UserPrincipalEx) == null)? string.Empty : ((UserPrincipalEx)r).TelephoneNumber
                        })
                        .ToList();
                    }
                }
            }
        }

        public static List<ActiveDirectoryUser> SearchUserByFirstAndLastName(string name)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var user = new UserPrincipalEx(context))
                {
                    user.DisplayName = name + "*";
                    user.Enabled = true;
                    using (var pS = new PrincipalSearcher())
                    {
                        pS.QueryFilter = user;
                        return pS.FindAll().Select(r => new ActiveDirectoryUser()
                        {
                            UserName = r.SamAccountName,
                            DisplayName = r.DisplayName,
                            EmailAddress = ((r as UserPrincipalEx) == null) ? string.Empty : ((UserPrincipalEx)r).EmailAddress,
                            ContactNumber = ((r as UserPrincipalEx) == null) ? string.Empty : ((UserPrincipalEx)r).TelephoneNumber
                        })
                        .ToList();
                    }
                }
            }
        }

        [Obsolete]
        public static List<ActiveDirectoryUser> SearchUserLINQ(string userName)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var userPrincipal = new UserPrincipalEx(context))
                {
                    userPrincipal.Enabled = true;
                    using (var searcher = new PrincipalSearcher(userPrincipal))
                    {
                        var query = from adUser in searcher.FindAll()
                                    where !string.IsNullOrEmpty(adUser.SamAccountName)
                                          && adUser.SamAccountName.Contains(userName)
                                    select new ActiveDirectoryUser()
                                               {
                                                   UserName = adUser.SamAccountName,
                                                   DisplayName = adUser.DisplayName,
                                                   EmailAddress = ((adUser as UserPrincipalEx) == null)
                                                                      ? string.Empty
                                                                      : ((UserPrincipal) adUser).EmailAddress,
                                                   ContactNumber = ((adUser as UserPrincipalEx) == null)
                                                                       ? string.Empty
                                                                       : ((UserPrincipalEx) adUser).TelephoneNumber
                                               };

                        return query.OrderBy(u => u.UserName).ToList();
                    }
                }
            }
        }

        public static bool IsUserInGroup(string userName, string groupName)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var user = new UserPrincipal(context))
                {
                    user.SamAccountName = userName;
                    using (var pS = new PrincipalSearcher())
                    {
                        pS.QueryFilter = user;
                        try
                        {
                            var result = pS.FindOne();
                            var userGroups = result.GetGroups();

                            //var x = userGroups.Select(i => i.Name).ToList();

                            return (userGroups.Select(g => g.Name).Contains(groupName));
                        }
                        catch (Exception ex)
                        {
                            return false;
                        }
                    }
                }
            }
        }

        public static bool IsValidUser(string userName)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var user = new UserPrincipal(context))
                {
                    user.SamAccountName = userName;
                    using (var pS = new PrincipalSearcher())
                    {
                        pS.QueryFilter = user;
                        try
                        {
                            var result = pS.FindOne();
                            return result != null;
                        }
                        catch (Exception)
                        {
                            return false;
                        }
                    }
                }
            }
        }

        [Obsolete]
        public static List<ActiveDirectoryUser> GetAllUsers()
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var userPrincipal = new UserPrincipalEx(context))
                {
                    userPrincipal.Enabled = true;
                    using (var searcher = new PrincipalSearcher(userPrincipal))
                    {
                        var query = from adUser in searcher.FindAll()
                                    where !string.IsNullOrEmpty(adUser.DisplayName)
                                          && !string.IsNullOrEmpty(adUser.SamAccountName)
                                    select new ActiveDirectoryUser()
                                               {
                                                   UserName = adUser.SamAccountName,
                                                   DisplayName = adUser.DisplayName,
                                                   EmailAddress = ((adUser as UserPrincipalEx) == null)
                                                                      ? string.Empty
                                                                      : ((UserPrincipal) adUser).EmailAddress,
                                                   ContactNumber = ((adUser as UserPrincipalEx) == null)
                                                                       ? string.Empty
                                                                       : ((UserPrincipalEx) adUser).TelephoneNumber
                                               };

                        return query.OrderBy(u => u.UserName).ToList();
                    }
                }
            }

        }

        [Obsolete]
        public static List<ActiveDirectoryUser> GetUsersByDomainAccounts(List<string> domainAccounts )
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var userPrincipal = new UserPrincipalEx(context))
                {
                    userPrincipal.Enabled = true;
                    using (var searcher = new PrincipalSearcher(userPrincipal))
                    {
                        var query = from adUser in searcher.FindAll()
                                    where !string.IsNullOrEmpty(adUser.DisplayName)
                                        && domainAccounts.Contains(adUser.SamAccountName)
                                    select new ActiveDirectoryUser()
                                    {
                                        UserName = adUser.SamAccountName,
                                        DisplayName = adUser.DisplayName,
                                        EmailAddress = ((adUser as UserPrincipalEx) == null)
                                                           ? string.Empty
                                                           : ((UserPrincipalEx)adUser).EmailAddress,
                                        ContactNumber = ((adUser as UserPrincipalEx) == null)
                                                            ? string.Empty
                                                            : ((UserPrincipalEx)adUser).TelephoneNumber
                                    };

                        return query.ToList();
                    }
                }
            }
        }

        public static ActiveDirectoryUser GetUser(string samAccountName)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var user = new UserPrincipalEx(context))
                {
                    user.Enabled = true;
                    user.SamAccountName = samAccountName;

                    using (var searcher = new PrincipalSearcher(user))
                    {
                        var adUser = searcher.FindOne();
                        if (adUser == null)
                            return null;

                        return new ActiveDirectoryUser()
                                   {
                                       UserName = adUser.SamAccountName,
                                       DisplayName = adUser.DisplayName,
                                       EmailAddress = ((adUser as UserPrincipalEx) == null)
                                                          ? string.Empty
                                                          : ((UserPrincipalEx) adUser).EmailAddress,
                                       ContactNumber = ((adUser as UserPrincipalEx) == null)
                                                           ? string.Empty
                                                           : ((UserPrincipalEx) adUser).TelephoneNumber
                                   };
                    }
                }
            }
        }

        [Obsolete]
        public static ActiveDirectoryUser GetUserEx(string userName)
        {
            using (var context = new PrincipalContext(ContextType.Domain, Domain))
            {
                using (var user = new UserPrincipal(context))
                {
                    user.SamAccountName = userName;
                    user.Enabled = true;
                    using (var pS = new PrincipalSearcher())
                    {
                        pS.QueryFilter = user;
                        var adUser = pS.FindOne() as UserPrincipal;
                        if (adUser != null)
                        {
                            return new ActiveDirectoryUser()
                                       {
                                           UserName = userName,
                                           DisplayName = adUser.DisplayName,
                                           EmailAddress = adUser.EmailAddress,
                                           ContactNumber = adUser.VoiceTelephoneNumber
                                       };
                        }
                    }
                }
            }

            return null;
        }

    }
}
