﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using Crossrail.IMS.Common;

namespace Crossrail.IMS.DataGateway.Repositories
{
    public class UserRepository : IUserRepository
    {
        public UserRepository()
        {
            Database.SetInitializer<MainContext>(null);
        }

        public int AddUser(User user, string userName)
        {
            using (var context = new MainContext())
            {
                //user already added
                if (context.Users.FirstOrDefault(u => u.UserName == user.UserName) != null)
                    return Constant.UserManagementErrorCode.USER_ALREADY_ADDED;

                var adUser = ActiveDirectoryHelper.GetUser(user.UserName);
                if (adUser != null)
                {
                    user.FullName = adUser.DisplayName;
                    user.Email = adUser.EmailAddress;
                    user.ContactNumber = adUser.ContactNumber;
                }

                context.Users.Add(user);

                if (user.ProposingItems != null)
                {
                    foreach (var item in user.ProposingItems)
                    {
                        context.UserProposingSiteProjects.Add(new UserProposingSiteProject()
                                                                  {
                                                                      UserID = user.ID,
                                                                      InnovationDataID = item.ID
                                                                  });
                    }
                }

                context.SaveChanges(userName);

                return user.ID;
            }
        }

        public List<Role> GetAllRoles()
        {
            using (var context = new MainContext())
            {
                return context.Roles.OrderBy(r => r.RoleName).ToList();
            }
        }

        public List<ActiveDirectoryUser> GetAllADUsers()
        {
            return ActiveDirectoryHelper.GetAllUsers();
        }

        public List<User> GetAllSystemUsers()
        {
            using (var context = new MainContext())
            {
                var systemUsers = (from user in context.Users.Include(c => c.ChampionFors)
                                   join role in context.Roles on user.RoleID equals role.ID
                                   select new
                                              {
                                                  ID = user.ID,
                                                  UserName = user.UserName,
                                                  RoleName = role.RoleName,
                                                  RoleID = role.ID,
                                                  IsDisabled = user.IsDisabled,
                                                  user.FullName,
                                                  user.Email,
                                                  user.ContactNumber,
                                                  user.ChampionFors
                                              }).OrderBy(u => u.UserName).ToList();

                var result = new List<User>();
                foreach (var systemUser in systemUsers)
                {
                    List<string> names = new List<string>();
                    string name = string.Empty;
                    if (systemUser.ChampionFors.Count > 0)
                    {
                        foreach (var item in systemUser.ChampionFors)
                        {
                            name = context.InnovationDatas.Where(c => c.ID == item.ChampionForID).Select(c => c.DataValue).FirstOrDefault();
                            if (name != null)
                            {
                                names.Add(name);
                            }
                        }
                    }
                    result.Add(new User()
                                   {
                                       ID = systemUser.ID,
                                       UserName = systemUser.UserName,
                                       FullName = systemUser.FullName,
                                       Email = systemUser.Email,
                                       ContactNumber = systemUser.ContactNumber,
                                       RoleName = systemUser.RoleName,
                                       RoleID = systemUser.RoleID,
                                       IsDisabled = systemUser.IsDisabled,
                                       ChampionFors = systemUser.ChampionFors,
                                       ChampionForNames = names.ToArray()
                                   });
                }

                return result;
            }
        }

        public User GetUser(int userID)
        {
            using (var context = new MainContext())
            {
                var systemUser = context.Users.Include(c => c.ChampionFors).FirstOrDefault(u => u.ID == userID);
                if (systemUser == null)
                    return null;

                var adUser = ActiveDirectoryHelper.GetUser(systemUser.UserName);
                if (adUser == null)
                    return null;

                systemUser.Email = adUser.EmailAddress;
                systemUser.FullName = adUser.DisplayName;
                systemUser.ContactNumber = adUser.ContactNumber;
                systemUser.ProposingItems = GetUserProposingItems(systemUser.ID);

                var role = context.Roles.FirstOrDefault(r => r.ID == systemUser.RoleID);
                if (role != null)
                    systemUser.RoleName = role.RoleName;

                return systemUser;
            }
        }

        public User GetUser(string userName)
        {
            using (var context = new MainContext())
            {
                var systemUser = context.Users.FirstOrDefault(u => u.UserName == userName);
                if (systemUser == null)
                    return null;

                var adUser = ActiveDirectoryHelper.GetUser(systemUser.UserName);
                if (adUser == null)
                    return null;

                systemUser.Email = adUser.EmailAddress;
                systemUser.FullName = adUser.DisplayName;
                systemUser.ContactNumber = adUser.ContactNumber;
                systemUser.ProposingItems = GetUserProposingItems(systemUser.ID);

                var role = context.Roles.FirstOrDefault(r => r.ID == systemUser.RoleID);
                if (role != null)
                    systemUser.RoleName = role.RoleName;

                return systemUser;
            }
        }

        private List<InnovationData> GetUserProposingItems(int userID)
        {
            using (var context = new MainContext())
            {
                var query = from item in context.UserProposingSiteProjects
                            where item.User.ID == userID
                            select item.InnovationData;

                return query.ToList();
            }
        }

        public int UpdateUser(User user, string userName)
        {
            using (var context = new MainContext())
            {
                var systemUser = context.Users.Include(c => c.ChampionFors).FirstOrDefault(u => u.ID == user.ID);
                if (systemUser != null)
                {
                    systemUser.RoleID = user.RoleID;
                    systemUser.IsDisabled = user.IsDisabled;
                }

                context.Entry(systemUser).State = EntityState.Modified;

                //delete old proposing items
                var query = from item in context.UserProposingSiteProjects
                            where item.UserID == user.ID
                            select item;

                foreach (var userProposingSiteProject in query)
                {
                    context.UserProposingSiteProjects.Remove(userProposingSiteProject);
                }

                //add new proposing items
                if (user.ProposingItems != null)
                {
                    foreach (var item in user.ProposingItems)
                    {
                        context.UserProposingSiteProjects.Add(new UserProposingSiteProject()
                        {
                            UserID = user.ID,
                            InnovationDataID = item.ID
                        });
                    }
                }

                //delete old champion for items
                if (systemUser.ChampionFors != null && systemUser.ChampionFors.Count > 0)
                {
                    systemUser.ChampionFors.Clear();
                }

                //add new champion for items
                if (user.ChampionFors != null)
                {
                    foreach (var item in user.ChampionFors)
                    {
                        systemUser.ChampionFors.Add(item);
                    }
                }

                return context.SaveChanges(userName);
            }
        }

        public bool IsValidUser(string userName)
        {
            using (var context = new MainContext())
            {
                var systemUser = context.Users.FirstOrDefault(user => user.UserName == userName && !user.IsDisabled);

                if (systemUser != null)
                    AuditLogin(userName);

                return systemUser != null;
            }
        }

        /// <summary>
        /// Audit the user login
        /// </summary>
        /// <param name="userName">The current login username</param>
        /// <returns>The affected record</returns>
        private int AuditLogin(string userName)
        {
            var auditLog = new AuditLog()
            {
                UserName = userName,
                EventDate = DateTime.Now,
                EventType = "Logged-in",
                TableName = "",
                ColumnName = ""
            };

            using (var context = new MainContext())
            {
                context.AuditLogs.Add(auditLog);

                return context.SaveChanges(userName);
            }
        }
    }
}