﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.Exchange.WebServices.Data;
using log4net;

namespace Crossrail.IMS.EmailProcessor {
    internal class EmailProcessor {
        private readonly ILog _log = LogManager.GetLogger("EmailProcessor");
        private readonly AppSettings _settings;
        public Dictionary<Item, string> ErroredEmails;
        private FolderId _archiveFolderId;
        private FolderId _rootFolder;
        private ExchangeService _service;

        public EmailProcessor() {
            _settings = new AppSettings();
            ErroredEmails = new Dictionary<Item, string>();
        }

        public Result Run() {
            Initialise();
            foreach (Item email in FindEmails()) {
                try {
                    LoadEmailBody(email);
                }
                catch {
                    ErroredEmails.Add(email, "Cannot load email body." + GetEmailDetails(email));
                    continue;
                }
                _log.Info(string.Format("Processing email item {0}", GetEmailDetails(email)));
                string str = string.Empty;
                string tempImportFolder;
                try {
                    tempImportFolder = ExtractEmailBodyAndSaveToTempImportFolder(email.Body);
                }
                catch {
                    ErroredEmails.Add(email, "Cannot save to file." + GetEmailDetails(email));
                    continue;
                }
                try {
                    MoveEmailToArchiveFolder(email);
                }
                catch {
                    ErroredEmails.Add(email, "Cannot move the email to archive folder." + GetEmailDetails(email));
                    continue;
                }
                string fileName;
                try {
                    fileName = MoveFile(tempImportFolder, _settings.InnovationSubmissionsRootFolder);
                }
                catch {
                    ErroredEmails.Add(email, "Cannot move the file." + GetEmailDetails(email));
                    continue;
                }
                SaveDetailsToDabase(email, fileName);
            }
            return (ErroredEmails).Any() ? Result.Fail : Result.Success;
        }

        private string MoveFile(string fullFilePath, string innovationSubmissionsRootFolder) {
            string newFullFilePath = string.Empty;
            RunWithTryCatchAndLog((() => {
                newFullFilePath = Path.Combine(innovationSubmissionsRootFolder, Path.GetFileName(fullFilePath));
                File.Move(fullFilePath, newFullFilePath);
            }), string.Format("An error occured while moving the file from TempImport folder to Innovation Submissions root folder. Temp File Name : {0}, New File Name : {1}", fullFilePath, newFullFilePath));
            return newFullFilePath;
        }

        private void SaveDetailsToDabase(Item email, string fileName) {
            RunWithTryCatchAndLog((() => new Db().SaveDetailsToDabase(email, fileName)), string.Format("An error occured while saving the details to database. {0}", GetEmailDetails(email)));
        }

        private string GetEmailDetails(Item email) {
            if (email == null)
                return "Email item is null.";
            try {
                return string.Format("Subject : {0} , Date Received : {1}", email.Subject, email.DateTimeReceived);
            }
            catch {
                return "Can not get email Subject and Date Time Received.";
            }
        }

        private void MoveEmailToArchiveFolder(Item item) {
            RunWithTryCatchAndLog((() => item.Move(_archiveFolderId)), string.Format("An error occured while moving the email to folder {0}. Email subject: {1}, Email date time received : {2}", _settings.ArchiveEmailFolderDisplayName, item.Subject, item.DateTimeReceived));
        }

        private void LoadEmailBody(Item item) {
            RunWithTryCatchAndLog((() => item.Load(new PropertySet(BasePropertySet.FirstClassProperties)
                {
                    RequestedBodyType = BodyType.Text
                })), string.Format("An error occured while loading the body text of the email : Email subject: {0}, Email date time received : {1}", item.Subject, item.DateTimeReceived));
        }

        private FindItemsResults<Item> FindEmails() {
            FindItemsResults<Item> findResults = null;
            RunWithTryCatchAndLog((() => findResults = _service.FindItems(_rootFolder, new SearchFilter.SearchFilterCollection(LogicalOperator.And, new SearchFilter[1]
                {
                    new SearchFilter.ContainsSubstring(ItemSchema.Subject, _settings.EmailTitle)
                }), new ItemView(_settings.EmailSearchResultSize))), string.Format("An error occured while searching emails. Subject line to search: {0}", _settings.EmailTitle));
            _log.InfoFormat("There are {0} emails containing {1} in subject line.", (findResults == null ? 0 : findResults.TotalCount), _settings.EmailTitle);
            return findResults;
        }

        private string ExtractEmailBodyAndSaveToTempImportFolder(MessageBody body) {
            string fullPath = string.Empty;
            RunWithTryCatchAndLog((() => {
                string bodyText = body.Text ?? string.Empty;
                fullPath = Path.Combine(_settings.InnovationSubmissionsTempImportFolder, Path.ChangeExtension(DateTime.Now.ToString(_settings.FileNameFormat), _settings.FileExtension));
                _log.InfoFormat("Saving the contents to file " + fullPath, new object[0]);
                _log.InfoFormat("Contents " + bodyText, new object[0]);
                using (FileStream fileStream = File.Create(fullPath)) {
                    byte[] fileContent = new UTF8Encoding(true).GetBytes(bodyText);
                    fileStream.Write(fileContent, 0, fileContent.Length);
                }
            }), string.Format("An error occured while saving the body of the email to file. Path: {0}", fullPath));
            return fullPath;
        }

        private void Initialise() {
            RunWithTryCatchAndLog((() => {
                _service = new ExchangeService(ExchangeVersion.Exchange2010);
                _service.Url = new Uri(_settings.EWSWebServiceUrl);
            }), string.Format("An error occured while initializing the Exchange Service. EWS Url : {0}", _settings.EWSWebServiceUrl));
            RunWithTryCatchAndLog((() => _service.Credentials = (ExchangeCredentials) new WebCredentials(_settings.DelegatedUser_UserName, _settings.DelegatedUser_Password, _settings.DelegatedUser_DomainName)),
                                  string.Format("An error occured while initializing the Exchange Service Credentials. DelegatedUser_UserName : {0}, DelegatedUser_Password : {1}, DelegatedUser_DomainName : {2}", _settings.DelegatedUser_UserName, _settings.DelegatedUser_Password,
                                                _settings.DelegatedUser_DomainName));
            RunWithTryCatchAndLog((() => _rootFolder = Folder.Bind(_service, new FolderId(WellKnownFolderName.Inbox, _settings.InnovationsMailBoxSmtpAddress)).Id), string.Format("An error occured while initializing the root email folder (Inbox) for  {0}", _settings.InnovationsMailBoxSmtpAddress));
            RunWithTryCatchAndLog((() => _archiveFolderId = FindFolderIdByDisplayName(_settings.ArchiveEmailFolderDisplayName)), string.Format("An error occured while searching the archive email folder in Inbox. Archive Email Folder Display Name {0}", _settings.ArchiveEmailFolderDisplayName));
            if (_archiveFolderId != null)
                return;
            string message = string.Format("Can not find Archive email folder in Inbox. Archive Email Folder Display Name {0}", _settings.ArchiveEmailFolderDisplayName);
            _log.Error(message);
            throw new ApplicationException(message);
        }

        public void RunWithTryCatchAndLog(Action action, string errorLogMessage) {
            try {
                action();
            }
            catch (Exception ex) {
                _log.Error(errorLogMessage, ex);
                throw;
            }
        }

        public FolderId FindFolderIdByDisplayName(string folderDisplayName) {
            foreach (Folder folder in Folder.Bind(_service, _rootFolder).FindFolders(new FolderView(100))) {
                if (folder.DisplayName == folderDisplayName)
                    return folder.Id;
            }
            return null;
        }
    }
}