﻿using System;
using System.ComponentModel;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using log4net;

namespace Crossrail.IMS.EmailProcessor {
    public class Service1 : ServiceBase {
        private readonly AppSettings _appSettings;
        private readonly ILog _log = LogManager.GetLogger(typeof (Service1));
        private bool _isProcessing;
        private DateTime? _notificationSentOn;
        private Timer _stateTimer;
        private TimerCallback _timerDelegate;
        private IContainer components = null;

        public Service1() {
            InitializeComponent();
            ServiceName = "Crossrail.IMS.EmailProcessor";
            CanStop = true;
            CanPauseAndContinue = false;
            AutoLog = true;
            _appSettings = new AppSettings();
        }

        protected override void OnStart(string[] args) {
            _log.LogInfo("Crossrail.IMS.EmailProcessor Service Starting", false);
            Start();
            _log.LogInfo("Crossrail.IMS.EmailProcessor Service Started", true);
        }

        protected override void OnStop() {
            _log.LogInfo("Crossrail.IMS.EmailProcessor Service Stopping", false);
            _stateTimer.Dispose();
            _log.LogInfo("Crossrail.IMS.EmailProcessor Service Stopped", true);
        }

        public void Start() {
            _timerDelegate = delegate { RunEmailProcessor(); };
            _stateTimer = new Timer(_timerDelegate, null, 5000, _appSettings.JobRunInterval);
        }

        private void RunEmailProcessor() {
            if (_isProcessing)
                return;
            _isProcessing = true;
            try {
                _log.Info("Running Email Processor.");
                var emailProcessor = new EmailProcessor();
                if (emailProcessor.Run() == Result.Fail) {
                    var stringBuilder = new StringBuilder();
                    stringBuilder.AppendLine(string.Format("Email Processor encountered {0} errors:", emailProcessor.ErroredEmails.Count));
                    int num = 1;
                    foreach (var keyValuePair in emailProcessor.ErroredEmails)
                        stringBuilder.AppendLine(string.Format("{0}) {1}", num++, keyValuePair.Value));
                    CheckAndSendErrorEmail((stringBuilder).ToString(), null);
                }
                _log.Info("Finished Running Email Processor.");
            }
            catch (Exception ex) {
                CheckAndSendErrorEmail("There was an error while running the processor.", ex);
            }
            finally {
                _isProcessing = false;
            }
        }

        private void CheckAndSendErrorEmail(string errorMessage, Exception exception) {
            bool sendEmail = false;
            if (!_notificationSentOn.HasValue)
                sendEmail = true;
            else if (DateTime.Now.Subtract(_notificationSentOn.Value).Minutes > _appSettings.NotificationIntervalInMinutes)
                sendEmail = true;
            if (sendEmail)
                _notificationSentOn = DateTime.Now;
            if (exception != null)
                _log.LogError(errorMessage, exception, sendEmail);
            else
                _log.LogError(errorMessage, sendEmail);
        }

        protected override void Dispose(bool disposing) {
            if (disposing && components != null)
                components.Dispose();
            base.Dispose(disposing);
        }

        private void InitializeComponent() {
            components = new Container();
            ServiceName = "Service1";
        }
    }
}