﻿using System;
using System.Configuration;
using System.Globalization;
using System.IO;
using Crossrail.IMS.Common;
using Crossrail.IMS.ImportInnovation.DataGatewayService;
using Crossrail.IMS.ImportInnovation.Model;
using Crossrail.IMS.ImportInnovation.Util;
using log4net;
using Innovation = Crossrail.IMS.ImportInnovation.DataGatewayService.Innovation;
using System.Collections.Generic;
using System.Linq;

namespace Crossrail.IMS.ImportInnovation
{
    // NOTE: You can use the "Rename" command on the "Refactor" menu to change the class name "Service1" in both code and config file together.
    public class ImportInnovationService : IImportInnovationService, IDisposable
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(ImportInnovationService).Name);
        /// <summary>
        /// CheckValidateFilename
        /// </summary>
        /// <param name="filename">filename of innovation</param>
        /// <returns>true: valid; false: invalid</returns>
        private bool CheckValidateFilename(string filename)
        {
            try
            {
                bool blnResult = false; //flag to indicate the filename is valid or not.
                if (filename.Length == 17)
                {
                    DateTime dtmFileName; //contains filename value in date format
                    int intNumericIdentifier = 0; //contain numeric identifier of filename
                    if (DateTime.TryParseExact(filename.Substring(0, 14), "yyyyMMddHHmmss", CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal, out dtmFileName) && int.TryParse(filename.Substring(14, 3), out intNumericIdentifier))
                    {
                        blnResult = true;
                    };
                }
                return blnResult;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Do Import Innovation from share folder
        /// </summary>
        public void DoImportInnovation()
        {
            try
            {
                string strInnovationFolder = CommonUtils.GetInnovationFolder(); //share folder contains innovation xml file.
                string[] strFilePaths = Directory.GetFiles(strInnovationFolder); //list file in the share folder.
                for (int i = 0; i < strFilePaths.Length; i++)
                {
                    //Validate Filename
                    if (Path.GetExtension(strFilePaths[i]) == ".xml" && CheckValidateFilename(Path.GetFileNameWithoutExtension(strFilePaths[i])))
                    {
                        //Do import one xml file.
                        DoImportOneInnovation(strFilePaths[i]);
                    }
                }
            }
            catch (Exception ex)
            {
                // Do something here to log exception or return status of importing.
            }
        }

        /// <summary>
        /// Do Import innovation from one XML file
        /// </summary>
        /// <param name="filename">file name of innovation</param>
        /// <returns>true: import is successfull; false: import is failed</returns>
        private bool DoImportOneInnovation(string filename)
        {
            string strProcessedFolder = CommonUtils.GetInnovationprocessedFolder(); //folder contains xml file processed successfully.
            string strFailedLoadsFolder = CommonUtils.GetInnovationFailedLoadsFolder(); //folder contains xml processed failed.
            if (File.Exists(filename))
            {
                try
                {
                    XmlInnovation _xmlInnovation = CommonUtils.DeserializeXml2Object<XmlInnovation>(filename, "ContactForm"); //object contains data of xml file
                    var errors = new Dictionary<string, string>();
                    Innovation _innovation = CovertInnovationFromXML(_xmlInnovation, ref errors); //valid innovation.
                    if (errors.Count > 0)
                    {
                        MoveFileProcessed(filename, strFailedLoadsFolder + "\\" + Path.GetFileName(filename));
                        var messages = string.Format("{0}|{1}", DateTime.Now.ToString(), filename);
                        var fields = errors.Select(c => string.Format("    {0} - {1}", c.Key, c.Value)).ToArray();
                        foreach (var item in fields)
                        {
                            messages += Environment.NewLine + item;
                        }
                        SendErrorMail(strFailedLoadsFolder + "\\" + Path.GetFileName(filename), messages);
                        log.Error(messages);
                        return false;
                    }
                    DataGatewayServiceClient _dataGatewayServiceClient = new DataGatewayServiceClient(); //proxy used to call Data gateway service
                    string strResult = _dataGatewayServiceClient.AddInnovation(_innovation, CommonUtils.GetImportServiceName()); //result after importing innovation 
                    string[] strResultArr = strResult.Split(new char[] { ':' }); //split string result to get error message or Innovation Ref
                    if (strResultArr[0] == "Success")
                    {
                        _innovation.InnovationRef = strResultArr[1];
                        MoveFileProcessed(filename, strProcessedFolder + "\\" + Path.GetFileName(filename));
                        //Send notification mail here . Only sent to email in XML tag
                        SendSuccessfulMail(_innovation);
                        //Send notification mail to SendCC in config file
                        SendToCCMail(_xmlInnovation.ProposingSiteProject, _xmlInnovation.Title);
                        //Send notification mail to user that has the same ProposingSiteProject
                        if (_innovation.ProposingSiteProjectID != null)
                        {
                            var innovationChampionEmails = _dataGatewayServiceClient.GetContactEmailByPSDId(_innovation.ProposingSiteProjectID.Value);
                            SendAlertToInnovationChampionRole(_xmlInnovation.ProposingSiteProject, _xmlInnovation.Title, innovationChampionEmails);
                        }

                        return true;
                    }
                    //Send warning mail here
                    MoveFileProcessed(filename, strFailedLoadsFolder + "\\" + Path.GetFileName(filename));
                    SendWarningMail(strFailedLoadsFolder + "\\" + Path.GetFileName(filename), strResultArr[1]);
                    return false;
                }
                catch (Exception ex)
                {
                    //Send warning mail here            
                    MoveFileProcessed(filename, strFailedLoadsFolder + "\\" + Path.GetFileName(filename));
                    SendWarningMail(strFailedLoadsFolder + "\\" + Path.GetFileName(filename), ex.Message);
                    log.Error(ex.Message, ex);
                    return false;
                }
            }
            return false;
        }

        /// <summary>
        /// Move file processed to appropriate folder.
        /// </summary>
        /// <param name="fileName">xml innovation input file</param>
        /// <param name="destinationFile">the folder contain xml file processed</param>
        private void MoveFileProcessed(string fileName, string destinationFile)
        {
            try
            {
                if (!Directory.Exists(Path.GetDirectoryName(destinationFile)))
                {
                    Directory.CreateDirectory(Path.GetDirectoryName(destinationFile));
                }
                File.Copy(fileName, destinationFile, true);
                File.Delete(fileName);
            }
            catch (Exception ex)
            {
                log.Error(ex.Message, ex);
            }
        }

        /// <summary>
        /// Covert data of Innovation from XML to Innovation object and validate it
        /// </summary>
        /// <param name="xmlInnovation">XMLInnovation object contained XML data</param>
        /// <returns>Innovation object</returns>
        private Innovation CovertInnovationFromXML(XmlInnovation xmlInnovation, ref Dictionary<string, string> errors)
        {
            Innovation _result = new Innovation
                                     {
                                         DateReceipt = DateTime.Now
                                     };

            RegexUtilities _regexUtilities = new RegexUtilities(); //regex utility used to validate email address.
            if (xmlInnovation.ContactEmail.Trim().Length <= 100 && xmlInnovation.ContactEmail.Trim().Length != 0 && _regexUtilities.CheckValidEmail(xmlInnovation.ContactEmail))
            {
                _result.ContactEmail = xmlInnovation.ContactEmail;
            }
            else
            {
                errors.Add("email", string.Format(CommonUtils.GetErrorMessage(), "email"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "email"));
            }

            if (xmlInnovation.ContactFirstName.Trim().Length == 0 || xmlInnovation.ContactFirstName.Trim().Length > 32)
            {
                errors.Add("firstname", string.Format(CommonUtils.GetErrorMessage(), "firstname"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "firstname"));
            }
            _result.ContactFirstName = xmlInnovation.ContactFirstName;

            if (xmlInnovation.ContactSurName.Trim().Length == 0 || xmlInnovation.ContactSurName.Trim().Length > 32)
            {
                errors.Add("lastname", string.Format(CommonUtils.GetErrorMessage(), "lastname"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "lastname"));
            }
            _result.ContactSurName = xmlInnovation.ContactSurName;

            if (xmlInnovation.ContactTelephone.Trim().Length == 0 || xmlInnovation.ContactTelephone.Trim().Length > 128)
            {
                errors.Add("telephone", string.Format(CommonUtils.GetErrorMessage(), "telephone"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "telephone"));
            }
            _result.ContactTelephone = xmlInnovation.ContactTelephone;

            if (xmlInnovation.Description.Trim().Length > 2000)
            {
                errors.Add("ideadescrip", string.Format(CommonUtils.GetErrorMessage(), "ideadescrip"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideadescrip"));
            }
            _result.Description = xmlInnovation.Description;

            if (xmlInnovation.InnovationContext.Trim().Length > 2000)
            {
                errors.Add("ideaopportunity", string.Format(CommonUtils.GetErrorMessage(), "ideaopportunity"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideaopportunity"));
            }
            _result.InnovationContext = xmlInnovation.InnovationContext;

            if (xmlInnovation.Title.Trim().Length == 0 || xmlInnovation.Title.Trim().Length > 256)
            {
                errors.Add("ideatitle", string.Format(CommonUtils.GetErrorMessage(), "ideatitle"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideatitle"));
            }
            _result.InnovationTitle = xmlInnovation.Title;

            if (xmlInnovation.ParentCompany.Length > 256)
            {
                errors.Add("ideaparentcomp", string.Format(CommonUtils.GetErrorMessage(), "ideaparentcomp"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideaparentcomp"));
            }
            _result.ParentCompany = xmlInnovation.ParentCompany;

            _result.ProposingSiteProjectID = this.GetInnovationDataId(xmlInnovation.ProposingSiteProject.Trim(), InnovationDataCategory.SiteProjectDoctorate);
            if (_result.ProposingSiteProjectID == -1 || xmlInnovation.ProposingSiteProject.Length > 256)
            {
                errors.Add("spd", string.Format(CommonUtils.GetErrorMessage(), "spd"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "spd"));
            }

            if (xmlInnovation.Who.Length > 1000)
            {
                errors.Add("ideainvolved", string.Format(CommonUtils.GetErrorMessage(), "ideainvolved"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideainvolved"));
            }
            _result.Who = xmlInnovation.Who;

            _result.IdeaTypeID = this.GetInnovationDataId(xmlInnovation.NeworShare.Trim(), InnovationDataCategory.IdeaType);
            if (_result.IdeaTypeID == -1 || xmlInnovation.NeworShare.Length > 256)
            {
                errors.Add("ideaneworshare", string.Format(CommonUtils.GetErrorMessage(), "ideaneworshare"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideaneworshare"));
            }

            _result.PrimaryBenefitID = this.GetInnovationDataId(xmlInnovation.Benefit.Trim(), InnovationDataCategory.PrimaryBenefit);
            if (_result.PrimaryBenefitID == -1 || xmlInnovation.Benefit.Length > 256)
            {
                errors.Add("ideabenefit", string.Format(CommonUtils.GetErrorMessage(), "ideabenefit"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideabenefit"));
            }

            _result.IdeaMaturityID = this.GetInnovationDataId(xmlInnovation.Maturity.Trim(), InnovationDataCategory.IdeaMaturity);
            if (_result.IdeaMaturityID == -1 || xmlInnovation.Maturity.Length > 256)
            {
                errors.Add("ideamaturity", string.Format(CommonUtils.GetErrorMessage(), "ideamaturity"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideamaturity"));
            }

            _result.InnovationSourceID = this.GetInnovationDataId(xmlInnovation.IdProcess.Trim(), InnovationDataCategory.InnovationSource);
            if (_result.InnovationSourceID == -1 || xmlInnovation.IdProcess.Length > 256)
            {
                errors.Add("ideaidprocess", string.Format(CommonUtils.GetErrorMessage(), "ideaidprocess"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideaidprocess"));
            }

            _result.ThemeID = this.GetInnovationDataId(xmlInnovation.Theme.Trim(), InnovationDataCategory.Theme);
            if (_result.ThemeID == -1 || xmlInnovation.Theme.Length > 256)
            {
                errors.Add("ideatheme", string.Format(CommonUtils.GetErrorMessage(), "ideatheme"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideatheme"));
            }
            int? themeId = _result.ThemeID;
            _result.SubThemeID = this.GetSubThemeIdByThemeId(xmlInnovation.Subtheme.Trim(), InnovationDataCategory.Theme, themeId);
            if (_result.SubThemeID == -1 || xmlInnovation.Subtheme.Length > 256)
            {
                errors.Add("ideasubtheme", string.Format(CommonUtils.GetErrorMessage(), "ideasubtheme"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideasubtheme"));
            }
            try
            {
                if (xmlInnovation.HoursSpent.Trim().Length > 0)
                {
                    _result.HoursDeveloping = int.Parse(xmlInnovation.HoursSpent);
                }
                else
                {
                    _result.HoursDeveloping = null;
                }
            }
            catch
            {
                errors.Add("ideahoursspent", string.Format(CommonUtils.GetErrorMessage(), "ideahoursspent"));
                //throw new Exception(string.Format(CommonUtils.GetErrorMessage(), "ideahoursspent"));
            }

            return _result;
        }

        /// <summary>
        /// Get Innovation Data ID
        /// </summary>
        /// <param name="text">value of innovation data</param>
        /// <param name="category">what innovation data category is used to get ID</param>
        /// <returns>ID of innovation data</returns>
        private int? GetInnovationDataId(string text, InnovationDataCategory category)
        {
            if (text.Trim().Length == 0)
            {
                return null;
            }
            int intResult = -1; //contain value of innovation data ID
            DataGatewayServiceClient _client = new DataGatewayServiceClient(); //proxy used to call services.
            InnovationData[] _listInnovationData = _client.GetInnovationDataByCategory(category); //list innovation data of current category            
            for (int i = 0; i < _listInnovationData.Length; i++)
            {
                if (_listInnovationData[i].DataValue.ToLower() == text.ToLower())
                {
                    return _listInnovationData[i].ID;
                }
            }
            return intResult;
        }

        private int? GetSubThemeIdByThemeId(string text, InnovationDataCategory category, int? themeId)
        {
            if (text.Trim().Length == 0 || themeId == null)
            {
                return null;
            }
            int intResult = -1; //contain value of innovation data ID
            DataGatewayServiceClient _client = new DataGatewayServiceClient(); //proxy used to call services.
            InnovationData[] _listInnovationData = _client.GetInnovationDataByCategory(category); //list innovation data of current category
            for (int i = 0; i < _listInnovationData.Length; i++)
            {
                var subThemeValue = _client.GetSubThemeByThemeID(themeId.Value);
                foreach (var subTheme in subThemeValue)
                {
                    if (subTheme.DataValue == text)
                    {
                        return subTheme.ID;
                    }
                }

            }
            return intResult;
        }
        /// <summary>
        /// Send Successful Mail
        /// </summary>
        /// <param name="innovation">innovation imported successfully</param>
        /// <returns>1: success; -1: failed</returns>
        private int SendSuccessfulMail(Innovation innovation)
        {
            try
            {
                string strEmailSubject = CommonUtils.GetNotificationEmailSubject();
                string strEmailBody = CommonUtils.GetNotificationEmailBody();
                string strSendFrom = CommonUtils.GetSendFrom();
                strEmailSubject = string.Format(strEmailSubject, innovation.InnovationRef);
                strEmailBody = string.Format(strEmailBody, innovation.ContactFirstName, innovation.InnovationRef, innovation.Description, innovation.InnovationContext);
                CommonUtils.SendMessage(innovation.ContactEmail, string.Empty, strSendFrom, strEmailSubject, strEmailBody, true);
                return 1;
            }
            catch
            {
                return -1;
            }
        }

        /// <summary>
        /// Send Warning Mail
        /// </summary>
        /// <param name="fileName">xml file used to import</param>
        /// <param name="errorMessage">error message is returned when processing xml file</param>
        /// <returns>1: success; -1: failed</returns>
        private int SendWarningMail(string fileName, string errorMessage)
        {
            try
            {
                string strEmailSubject = string.Empty; //subject of email will be sent
                string strEmailBody = string.Empty; //content will be sent 
                string strSendTo = string.Empty; //receiver email address
                string strSendCC = string.Empty; // receiver in CC list
                string strSendFrom = string.Empty; // sender email address
                strEmailSubject = CommonUtils.GetAlertEmailSubject();
                strEmailBody = CommonUtils.GetAlertEmailBody();
                strSendTo = CommonUtils.GetContactEmail();
                strSendCC = CommonUtils.GetSendCC();
                strSendFrom = CommonUtils.GetSendFrom();
                strEmailBody = string.Format(strEmailBody, Path.GetFileName(fileName), CommonUtils.GetServerNameRunImport(), errorMessage);
                CommonUtils.SendMessage(strSendTo, strSendCC, strSendFrom, strEmailSubject, strEmailBody);
                return 1;
            }
            catch
            {
                return -1;
            }
        }

        private int SendErrorMail(string fileName, string errorMessage)
        {
            try
            {
                string strEmailSubject = CommonUtils.GetAlertEmailSubject();
                string strSendTo = CommonUtils.GetErrorEmail();
                string strSendCC = CommonUtils.GetSendCC();
                string strSendFrom = CommonUtils.GetSendFrom();
                CommonUtils.SendMessage(strSendTo, strSendCC, strSendFrom, strEmailSubject, errorMessage, false, fileName);
                return 1;
            }
            catch
            {
                return -1;
            }
        }

        private int SendAlertToInnovationChampionRole(string psd, string innovationTitle, string sendCC)
        {
            try
            {
                string strEmailSubject = string.Empty;
                string strEmailBody = string.Empty;
                string strSendTo = string.Empty;
                string strSendCC = string.Empty;
                string strSendFrom = string.Empty;
                strEmailSubject = string.Format(Crossrail.IMS.ImportInnovation.Resource.InnovationSubmissionAlertSubject, psd);
                strEmailBody = Crossrail.IMS.ImportInnovation.Resource.InnovationSubmissionAlertBody;
                strSendTo = sendCC;
                //strSendCC = sendCC;
                strSendFrom = CommonUtils.GetSendFrom();
                strEmailBody = string.Format(strEmailBody, innovationTitle, psd);
                CommonUtils.SendMessage(strSendTo, strSendCC, strSendFrom, strEmailSubject, strEmailBody);
                return 1;
            }
            catch
            {
                return -1;
            }
        }

        private int SendToCCMail(string psd, string innovationTitle)
        {
            try
            {
                string strEmailSubject = string.Empty;
                string strEmailBody = string.Empty;
                string strSendTo = string.Empty;
                string strSendCC = string.Empty;
                string strSendFrom = string.Empty;
                strEmailSubject = string.Format(Crossrail.IMS.ImportInnovation.Resource.InnovationSubmissionAlertSubject, psd);
                strEmailBody = Crossrail.IMS.ImportInnovation.Resource.InnovationSubmissionAlertBody;
                strSendTo = CommonUtils.GetSendCC();
                strSendFrom = CommonUtils.GetSendFrom();
                strEmailBody = string.Format(strEmailBody, innovationTitle, psd);
                CommonUtils.SendMessage(strSendTo, strSendCC, strSendFrom, strEmailSubject, strEmailBody);
                log.Info("Send mail:" + strSendTo);
                return 1;
            }
            catch
            {
                return -1;
            }
        }

        /// <summary>
        /// Dispose the resource
        /// </summary>
        public void Dispose()
        {
            // Use SupressFinalize in case a subclass 
            // of this type implements a finalizer.
            GC.SuppressFinalize(this);
        }


    }
}
