﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using Crossrail.IMS.Common;
using Crossrail.IMS.WebApp.DataGatewayService;
using Crossrail.IMS.WebApp.Models;

namespace Crossrail.IMS.WebApp.Controllers
{
    public class BaseController : Controller
    {
        protected override void OnAuthorization(AuthorizationContext filterContext)
        {
            base.OnAuthorization(filterContext);
            
            if (!User.Identity.IsAuthenticated)
            {
                if (Authenticate())
                {
                    ReturnPreviousPage(filterContext);
                }
            }
            else
            {
                if (Session[Constant.SessionVariables.UserDisplayName] == null)
                {
                    if (Authenticate())
                    {
                        ReturnPreviousPage(filterContext);
                    }
                }
                CheckSessionTimedOut();
            }
        }

        private bool Authenticate()
        {
            string domain = "";
            string userName = Request.ServerVariables["LOGON_USER"];
            string allowedDomain = ConfigurationManager.AppSettings["Domain"];
            string adMembershipGroupName = ConfigurationManager.AppSettings["ADMembershipGroupName"];

            if (userName.Contains(@"\"))
            {
                domain = userName.Substring(0, userName.IndexOf(@"\"));
                userName = userName.Substring(userName.IndexOf(@"\") + 1);
            }

            if (domain.ToUpper() != allowedDomain.ToUpper()) //is in another domain?
                return false;//

            bool isValidUser = ActiveDirectoryHelper.IsValidUser(userName);
            if (!isValidUser)
                return false;

            bool isUserInADGroup = ActiveDirectoryHelper.IsUserInGroup(userName, adMembershipGroupName);
            if (!isUserInADGroup)
                return false;

            var client = new DataGatewayServiceClient();
            var user = client.GetUserByUserName(userName);

            if (user != null && !user.IsDisabled)
            {
                FormsAuthentication.SetAuthCookie(userName, false);
                Session[Constant.SessionVariables.UserDisplayName] = user.FullName;
                Session[Constant.SessionVariables.UserRole] = user.RoleName;
                Session[Constant.SessionVariables.UserID] = user.ID;
                
                client.AuditLoginUser(userName);

                return true;
            }

            return false;
        }

        private void ReturnPreviousPage(AuthorizationContext filterContext)
        {
            string returnUrl = Request.QueryString["ReturnUrl"];
            if (!string.IsNullOrEmpty(returnUrl))
                filterContext.Result = new RedirectResult(returnUrl);
            else
                filterContext.Result = new RedirectResult("~/Home");
        }

        /// <summary>
        /// If session has expired, get user's display name & role again
        /// </summary>
        private void CheckSessionTimedOut()
        {
            if (Session != null)// check if session is supported?
            {
                if (Session.IsNewSession) //a new session id was generated?
                {
                    // If it says it is a new session, but an existing cookie exists, then it must have timed out
                    string cookie = Request.Headers["Cookie"];
                    if (!string.IsNullOrEmpty(cookie) && cookie.Contains(Constant.SessionVariables.SessionID))
                    {
                        if (User.Identity.IsAuthenticated)
                        {
                            var userName = User.Identity.Name;
                            if (userName.Contains(@"\"))
                                userName = userName.Substring(userName.IndexOf(@"\") + 1);

                            var client = new DataGatewayServiceClient();
                            var user = client.GetUserByUserName(userName);
                            if (user != null && !user.IsDisabled)
                            {
                                Session[Constant.SessionVariables.UserDisplayName] = user.FullName;
                                Session[Constant.SessionVariables.UserRole] = user.RoleName;
                                Session[Constant.SessionVariables.UserID] = user.ID;
                            }
                        }
                    }
                }
            }
        }
    }

    public static class Extensions
    {
        public static List<SelectListItem> ToMvcSelectList(this IEnumerable<InnovationData> source)
        {
            return source.Select(item => new SelectListItem()
                                             {
                                                 Value = item.ID.ToString(), Text = item.DataValue
                                             }).ToList();
        }

        public static List<SelectListItem> ToMvcSelectList(this IEnumerable<SubTheme> source )
        {
            return source.Select(item => new SelectListItem()
                                             {
                                                 Value = item.ID.ToString(),
                                                 Text = item.DataValue
                                             }).ToList();
        }

        public static InnovationModel ToModel(this Innovation innovation)
        {
            return new InnovationModel()
                       {
                           ID = innovation.ID,
                           InnovationRef = innovation.InnovationRef,
                           InnovationTitle = innovation.InnovationTitle,
                           ProposingItem = innovation.ProposingItem == null
                                               ? string.Empty
                                               : innovation.ProposingItem.DataValue,
                           ThemeName = innovation.Theme == null
                                           ? string.Empty
                                           : innovation.Theme.DataValue,
                           SubThemeName = innovation.SubTheme == null
                                              ? string.Empty
                                              : innovation.SubTheme.DataValue,
                           CurrentIRLName = innovation.CurrentIRL == null
                                                ? string.Empty
                                                : innovation.CurrentIRL.DataValue,

                           SubmittedForCompetitionDate = innovation.SubmittedForCompetitionDate,
                           CurrentStage = innovation.CurrentStage == null
                                              ? string.Empty
                                              : innovation.CurrentStage.DataValue,
                           NextSteps = innovation.NextSteps
                       };
        }

        public static List<InnovationModel> ToModelList(this IEnumerable<Innovation> source)
        {
            return source.Select(item => item.ToModel()).ToList();
        }
    }

    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public class IMSAuthorizeAttribute: AuthorizeAttribute
    {
        protected override bool AuthorizeCore(HttpContextBase httpContext)
        {
            if(HttpContext.Current.Session[Constant.SessionVariables.UserRole] != null)
            {
                string userRole = HttpContext.Current.Session[Constant.SessionVariables.UserRole].ToString();
                string[] roles = Roles.Split(new string[] {","}, StringSplitOptions.RemoveEmptyEntries);

                if (roles.Contains(userRole))
                    return true;
            }

            return false;
        }

        protected override void HandleUnauthorizedRequest(AuthorizationContext filterContext)
        {
            filterContext.Result = new RedirectToRouteResult(new RouteValueDictionary()
                                                                 {
                                                                     {"action","NotAuthorized"},
                                                                     {"controller","UserManagement"}
                                                                 });

        }

        public IMSAuthorizeAttribute()
        {
            
        }

        public IMSAuthorizeAttribute(params string[] roles)
        {
            foreach (var role in roles)
            {
                Roles += role + ",";
            }
        }
    }
}
