﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Kendo.Mvc.UI;
using Kendo.Mvc.Extensions;
using Crossrail.IMS.WebApp.DataGatewayService;
using Crossrail.IMS.Common;
using Crossrail.IMS.WebApp.Models;
using System.Reflection;
using System.IO;

namespace Crossrail.IMS.WebApp.Controllers
{
    [ValidateInput(false)]
    [OutputCache(NoStore = true, Duration = 0, VaryByParam = "*")]    
    public class DataManagementController : BaseController
    {
        //
        // GET: /DataManagement/

        public IDataGatewayService DataService
        {
            get
            {
                return new DataGatewayService.DataGatewayServiceClient();
            }
            set { }
        }

        public string UserNameLog
        {
            get
            {
                return Request.ServerVariables["LOGON_USER"];
            }
        }

        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator)]
        public ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Map the the grid model object to Crossrail.IMS.WebApp.DataGatewayService.InnovationData object 
        /// </summary>
        /// <param name="model">The grid model</param>
        /// <returns>The InnovationData object</returns>
        private InnovationData ConvertToInnovationDataEntity(IInnovationDataModel model)
        {
            if (model == null)
            {
                return new InnovationData();
            }

            return new InnovationData() { ID = model.ID, DataValue = model.DataValue, Category = model.Category };

        }

        /// <summary>
        /// Update detail of Innovation data
        /// </summary>
        /// <param name="data"></param>
        /// <param name="category"></param>
        /// <returns></returns>
        private ModelStateDictionary UpdateInnovationData(object data, InnovationDataCategory category)
        {
            if (data != null && ModelState.IsValid)
            {
                try
                {
                    Type type = data.GetType();
                    PropertyInfo propertyInfo_DataValue = type.GetProperty("DataValue");
                    string dataValue = string.Empty;
                    if (propertyInfo_DataValue.CanRead)
                    {
                        dataValue = TrimString(propertyInfo_DataValue.GetValue(data, null).ToString());
                    }

                    PropertyInfo propInfo_ID = type.GetProperty("ID");
                    int innovationId = 0;
                    if (propInfo_ID.CanRead)
                    {
                        innovationId = int.Parse(propInfo_ID.GetValue(data, null).ToString());
                    }
                    if (!DataService.IsInnovationDataValueExist(innovationId, dataValue, category))
                    {
                        int returnValue = DataService.UpdateInnovationData(new InnovationData() { ID = innovationId, DataValue = dataValue, Category = (int)category }, UserNameLog);
                        if (returnValue == -1)
                        {
                            ModelState.AddModelError("DataValue", "not_consistant");
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("DataValue", Common.Constant.DataManagementErrorCode.ErrorMessageDuplicateName);
                    }
                }
                catch (Exception ex)
                {
                    ModelState.AddModelError("Exeption", ex.Message);
                }
            }

            return ModelState;
        }

        /// <summary>
        /// Create new Innovation data
        /// </summary>
        /// <param name="request"></param>
        /// <param name="data"></param>
        /// <param name="category"></param>
        /// <returns></returns>
        private ActionResult CreateInnovationData(DataSourceRequest request, object data, InnovationDataCategory category)
        {
            JsonResult result = null;
            if (data != null && ModelState.IsValid)
            {
                try
                {
                    Type type = data.GetType();
                    PropertyInfo propertyInfo_DataValue = type.GetProperty("DataValue");
                    string dataValue = string.Empty;
                    if (propertyInfo_DataValue.CanRead)
                    {
                        dataValue = TrimString(propertyInfo_DataValue.GetValue(data, null).ToString());
                    }
                    if (!DataService.IsInnovationDataValueExist(0, dataValue, category))
                    {
                        int id = DataService.AddInnovationData(new InnovationData() { DataValue = dataValue, Category = (int)category }, UserNameLog);
                        PropertyInfo propInfo_ID = type.GetProperty("ID");
                        propInfo_ID.SetValue(data, id, null);
                        result = Json(new[] { data }.ToDataSourceResult(request, ModelState));
                    }
                    else
                    {
                        ModelState.AddModelError("DataValue", Common.Constant.DataManagementErrorCode.ErrorMessageDuplicateName);
                        result = Json(ModelState.ToDataSourceResult());
                    }

                }
                catch (Exception ex)
                {
                    ModelState.AddModelError("Exception", ex.Message);
                    result = Json(ModelState.ToDataSourceResult());
                }
            }
            return result;
        }

        /// <summary>
        /// Handle the delete action
        /// </summary>
        /// <param name="innovationDataId"></param>
        /// <returns></returns>
        private ModelStateDictionary DeleteInnovationData(int innovationDataId)
        {
            try
            {
                DataService.DeleteInnovationData(innovationDataId, UserNameLog);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("DataValue", "delete");
            }

            return ModelState;
        }

        /// <summary>
        /// Get the list innovation grid model by category
        /// </summary>
        /// <param name="innovationType">The InnovationData category</param>
        /// <returns>List of innovation grid model</returns>
        private IEnumerable<IInnovationDataModel> GetInnovationDatas(InnovationDataCategory innovationType)
        {
            string sessionId = string.Empty;

            IEnumerable<IInnovationDataModel> listInnovationData = null;
            switch (innovationType)
            {
                case InnovationDataCategory.SiteProjectDoctorate:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new DirectorateModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.Theme:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new ThemeModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.Area:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new AreaModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.Stage:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new StageModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.IRL:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new IrlModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.HowCanWeHelp:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new HowCanWeHelpModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.IndividualOrSharedInnovation:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new IndividualOrSharedInnovationModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.BenefitClearlyUnderstood:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new BenefitClearlyUnderstoodModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.SuitablyDevelopedToShare:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new SuitablyDevelopedToShareModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.IdeaType:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new IdeaTypeModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.IdeaMaturity:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new IdeaMaturityModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.PrimaryBenefit:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new PrimaryBenefitModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.InnovationSource:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new InnovationSourceModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
                case InnovationDataCategory.AssignedInnovationTeamMember:
                    listInnovationData = DataService.GetInnovationDataByCategory(innovationType)
                        .Select(i => new DirectorateModel() { ID = i.ID, DataValue = i.DataValue, Category = i.Category }).OrderBy(i => i.DataValue).ToList();
                    break;
            }

            return listInnovationData;
        }

        private string TrimString(string inputString)
        {            
            return String.IsNullOrEmpty(inputString) ? string.Empty : inputString.Trim();
        }

        #region Directorate
        public ActionResult GridDirectorate_Read([DataSourceRequest] DataSourceRequest request)
        {
            JsonResult result;
            try
            {
                IEnumerable<DirectorateModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.SiteProjectDoctorate) as IEnumerable<DirectorateModel>;

                result = Json(listInnovationData.ToDataSourceResult(request));
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("DataValue", ex.Message);
                result = Json(ModelState.ToDataSourceResult());
            }
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridDirectorate_Update([DataSourceRequest] DataSourceRequest request, DirectorateModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.SiteProjectDoctorate);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridDirectorate_Delete([DataSourceRequest] DataSourceRequest request, DirectorateModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridDirectorate_Create([DataSourceRequest] DataSourceRequest request, DirectorateModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.SiteProjectDoctorate);
        }
        #endregion

        #region Theme
        public ActionResult GridTheme_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<ThemeModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.Theme) as IEnumerable<ThemeModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridTheme_Update([DataSourceRequest] DataSourceRequest request, ThemeModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.Theme);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridTheme_Delete([DataSourceRequest] DataSourceRequest request, ThemeModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridTheme_Create([DataSourceRequest] DataSourceRequest request, ThemeModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.Theme);
        }
        #endregion

        #region Area
        public ActionResult GridArea_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<AreaModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.Area) as IEnumerable<AreaModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridArea_Update([DataSourceRequest] DataSourceRequest request, AreaModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.Area);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridArea_Delete([DataSourceRequest] DataSourceRequest request, AreaModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridArea_Create([DataSourceRequest] DataSourceRequest request, AreaModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.Area);
        }
        #endregion

        #region Stage
        public ActionResult GridStage_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<StageModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.Stage) as IEnumerable<StageModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridStage_Update([DataSourceRequest] DataSourceRequest request, StageModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.Stage);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridStage_Delete([DataSourceRequest] DataSourceRequest request, StageModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridStage_Create([DataSourceRequest] DataSourceRequest request, StageModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.Stage);
        }
        #endregion

        #region Irl
        public ActionResult GridIrl_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<IrlModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.IRL) as IEnumerable<IrlModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIrl_Update([DataSourceRequest] DataSourceRequest request, IrlModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.IRL);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIrl_Delete([DataSourceRequest] DataSourceRequest request, IrlModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIrl_Create([DataSourceRequest] DataSourceRequest request, IrlModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.IRL);
        }
        #endregion

        #region HowCanWeHelp
        public ActionResult GridHowCanWeHelp_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<HowCanWeHelpModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.HowCanWeHelp) as IEnumerable<HowCanWeHelpModel>;


            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridHowCanWeHelp_Update([DataSourceRequest] DataSourceRequest request, HowCanWeHelpModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.HowCanWeHelp);
            return Json(ModelState.ToDataSourceResult());
        }


        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridHowCanWeHelp_Delete([DataSourceRequest] DataSourceRequest request, HowCanWeHelpModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridHowCanWeHelp_Create([DataSourceRequest] DataSourceRequest request, HowCanWeHelpModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.HowCanWeHelp);
        }
        #endregion

        #region IndividualOrSharedInnovation
        public ActionResult GridIndividualOrSharedInnovation_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<IndividualOrSharedInnovationModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.IndividualOrSharedInnovation) as IEnumerable<IndividualOrSharedInnovationModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIndividualOrSharedInnovation_Update([DataSourceRequest] DataSourceRequest request, IndividualOrSharedInnovationModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.IndividualOrSharedInnovation);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIndividualOrSharedInnovation_Delete([DataSourceRequest] DataSourceRequest request, IndividualOrSharedInnovationModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIndividualOrSharedInnovation_Create([DataSourceRequest] DataSourceRequest request, IndividualOrSharedInnovationModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.IndividualOrSharedInnovation);
        }
        #endregion

        #region BenefitClearlyUnderstood
        public ActionResult GridBenefitClearlyUnderstood_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<BenefitClearlyUnderstoodModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.BenefitClearlyUnderstood) as IEnumerable<BenefitClearlyUnderstoodModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridBenefitClearlyUnderstood_Update([DataSourceRequest] DataSourceRequest request, BenefitClearlyUnderstoodModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.BenefitClearlyUnderstood);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridBenefitClearlyUnderstood_Delete([DataSourceRequest] DataSourceRequest request, BenefitClearlyUnderstoodModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridBenefitClearlyUnderstood_Create([DataSourceRequest] DataSourceRequest request, BenefitClearlyUnderstoodModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.BenefitClearlyUnderstood);
        }
        #endregion

        #region SuitablyDevelopedToShare
        public ActionResult GridSuitablyDevelopedToShare_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<SuitablyDevelopedToShareModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.SuitablyDevelopedToShare) as IEnumerable<SuitablyDevelopedToShareModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridSuitablyDevelopedToShare_Update([DataSourceRequest] DataSourceRequest request, SuitablyDevelopedToShareModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.SuitablyDevelopedToShare);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridSuitablyDevelopedToShare_Delete([DataSourceRequest] DataSourceRequest request, SuitablyDevelopedToShareModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridSuitablyDevelopedToShare_Create([DataSourceRequest] DataSourceRequest request, SuitablyDevelopedToShareModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.SuitablyDevelopedToShare);
        }
        #endregion

        #region SubTheme

        public ActionResult SubTheme_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<SubThemModel> listSubTheme = DataService.GetSubThemes()
                .Select(st => new SubThemModel { SubThemeId = st.ID, SubThemeValue = st.DataValue, Theme = st.InnovationData }).OrderBy(s => s.SubThemeValue).ToList();
            JsonResult result = Json(listSubTheme.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SubTheme_Update([DataSourceRequest] DataSourceRequest request, SubThemModel subTheme)
        {
            if (subTheme != null && ModelState.IsValid)
            {
                try
                {
                    if (!DataService.IsSubThemeNameExist(subTheme.SubThemeId, subTheme.SubThemeValue))
                    {
                        SubTheme sub = new DataGatewayService.SubTheme()
                        {
                            ID = subTheme.SubThemeId,
                            DataValue = TrimString(subTheme.SubThemeValue),
                            InnovationData = subTheme.Theme
                        };
                        int returnValue = DataService.UpdateSubTheme(sub, UserNameLog);
                        if (returnValue == -1)
                        {
                            ModelState.AddModelError("DataValue", "not_consistant");
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("SubThemeValue", Common.Constant.DataManagementErrorCode.ErrorMessageDuplicateName);
                    }
                }
                catch (Exception ex)
                {
                    ModelState.AddModelError("Exception", ex.Message);
                }

            }
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SubTheme_Delete([DataSourceRequest] DataSourceRequest request, SubThemModel subTheme)
        {
            if (subTheme != null)
            {
                try
                {
                    DataService.DeleteSubTheme(subTheme.SubThemeId, UserNameLog);
                }
                catch (Exception ex)
                {
                    ModelState.AddModelError("DataValue", "delete");
                }
            }

            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SubTheme_Create([DataSourceRequest] DataSourceRequest request, SubThemModel subTheme)
        {
            JsonResult result = null;
            if (subTheme != null && ModelState.IsValid)
            {
                try
                {
                    if (!DataService.IsSubThemeNameExist(0, subTheme.SubThemeValue))
                    {
                        //InnovationData innovationData = DataService.GetInnovationDataById(subTheme.Theme.ID);
                        if (subTheme.Theme.ID == 0)
                        {
                            subTheme.Theme = DataService.GetInnovationDataByCategory(InnovationDataCategory.Theme).OrderBy(i => i.DataValue).First();
                        }
                        SubTheme sub = new DataGatewayService.SubTheme() { DataValue = TrimString(subTheme.SubThemeValue), InnovationData = subTheme.Theme };
                        int id = DataService.AddSubTheme(sub, UserNameLog);
                        subTheme.SubThemeId = id;

                        result = Json(new[] { subTheme }.ToDataSourceResult(request, ModelState));
                    }
                    else
                    {
                        ModelState.AddModelError("SubThemeValue", Common.Constant.DataManagementErrorCode.ErrorMessageDuplicateName);
                        result = Json(ModelState.ToDataSourceResult());
                    }

                }
                catch (Exception ex)
                {
                    ModelState.AddModelError("Exception", ex.Message);
                    result = Json(ModelState.ToDataSourceResult());
                }
            }
            return result;
        }

        public JsonResult GetThemesForDropDown()
        {
            IEnumerable<InnovationData> listTheme = DataService.GetInnovationDataByCategory(InnovationDataCategory.Theme).OrderBy(i => i.DataValue).ToList();
            JsonResult result = Json(listTheme, JsonRequestBehavior.AllowGet);


            //string tempt = RenderPartialViewToString("~/Views/Shared/EditorTemplates/GridThemeForeignKey.ascx", null);
            return result;
        }

        #endregion

        #region IdeaMaturity
        public ActionResult GridIdeaMaturity_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<IdeaMaturityModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.IdeaMaturity) as IEnumerable<IdeaMaturityModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIdeaMaturity_Update([DataSourceRequest] DataSourceRequest request, IdeaMaturityModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.IdeaMaturity);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIdeaMaturity_Delete([DataSourceRequest] DataSourceRequest request, IdeaMaturityModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIdeaMaturity_Create([DataSourceRequest] DataSourceRequest request, IdeaMaturityModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.IdeaMaturity);
        }
        #endregion

        #region IdeaType
        public ActionResult GridIdeaType_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<IdeaTypeModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.IdeaType) as IEnumerable<IdeaTypeModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIdeaType_Update([DataSourceRequest] DataSourceRequest request, IdeaTypeModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.IdeaType);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIdeaType_Delete([DataSourceRequest] DataSourceRequest request, IdeaTypeModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridIdeaType_Create([DataSourceRequest] DataSourceRequest request, IdeaTypeModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.IdeaType);
        }
        #endregion

        #region InnovationSource
        public ActionResult GridInnovationSource_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<InnovationSourceModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.InnovationSource) as IEnumerable<InnovationSourceModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridInnovationSource_Update([DataSourceRequest] DataSourceRequest request, InnovationSourceModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.InnovationSource);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridInnovationSource_Delete([DataSourceRequest] DataSourceRequest request, InnovationSourceModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridInnovationSource_Create([DataSourceRequest] DataSourceRequest request, InnovationSourceModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.InnovationSource);
        }
        #endregion

        #region PrimaryBenefit
        public ActionResult GridPrimaryBenefit_Read([DataSourceRequest] DataSourceRequest request)
        {
            IEnumerable<PrimaryBenefitModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.PrimaryBenefit) as IEnumerable<PrimaryBenefitModel>;

            JsonResult result = Json(listInnovationData.ToDataSourceResult(request));
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridPrimaryBenefit_Update([DataSourceRequest] DataSourceRequest request, PrimaryBenefitModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.PrimaryBenefit);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridPrimaryBenefit_Delete([DataSourceRequest] DataSourceRequest request, PrimaryBenefitModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridPrimaryBenefit_Create([DataSourceRequest] DataSourceRequest request, PrimaryBenefitModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.PrimaryBenefit);
        }
        #endregion

        #region Assigned Innovation Team Member
        public ActionResult GridAssignedInnovationTeamMember_Read([DataSourceRequest] DataSourceRequest request)
        {
            JsonResult result;
            try
            {
                IEnumerable<DirectorateModel> listInnovationData = GetInnovationDatas(InnovationDataCategory.AssignedInnovationTeamMember) as IEnumerable<DirectorateModel>;

                result = Json(listInnovationData.ToDataSourceResult(request));
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("DataValue", ex.Message);
                result = Json(ModelState.ToDataSourceResult());
            }
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridAssignedInnovationTeamMember_Update([DataSourceRequest] DataSourceRequest request, DirectorateModel data)
        {
            ModelStateDictionary modelState = UpdateInnovationData(data, InnovationDataCategory.AssignedInnovationTeamMember);
            return Json(ModelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridAssignedInnovationTeamMember_Delete([DataSourceRequest] DataSourceRequest request, DirectorateModel data)
        {
            ModelStateDictionary modelState = DeleteInnovationData(data.ID);
            return Json(modelState.ToDataSourceResult());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GridAssignedInnovationTeamMember_Create([DataSourceRequest] DataSourceRequest request, DirectorateModel data)
        {
            return CreateInnovationData(request, data, InnovationDataCategory.AssignedInnovationTeamMember);
        }
        #endregion
    }
}
