﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Crossrail.IMS.Common;
using Crossrail.IMS.WebApp.Models;
using Crossrail.IMS.WebApp.DataGatewayService;
using Kendo.Mvc.UI;
using Kendo.Mvc.Extensions;
using Crossrail.IMS.WebApp.Filters;

namespace Crossrail.IMS.WebApp.Controllers
{
    public class UserManagementController : BaseController
    {
        public string UserNameLog
        {
            get
            {
                return Request.ServerVariables["LOGON_USER"];
            }
        }

        //
        // GET: /UserManagement/

        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator, Constant.Role.InnovationTeam)]
        public ActionResult Index()
        {
            return RedirectToAction("UserList");
        }

        [DisableCache]
        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator, Constant.Role.InnovationTeam)]
        public ActionResult GetUserList([DataSourceRequest] DataSourceRequest request)
        {
            return Json(GetAllSystemUsers().ToDataSourceResult(request));
        }

        public JsonResult SearchADUser(string text)
        {
            var result = ActiveDirectoryHelper.SearchUser(text);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        public JsonResult SearchADUserByFirstName(string text)
        {
            var result = ActiveDirectoryHelper.SearchUserByFirstAndLastName(text);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        private List<ProposingItem> GetAllProposingItems()
        {
            var result = new List<ProposingItem>();
            var client = new DataGatewayServiceClient();
            var data = client.GetInnovationDataByCategory(InnovationDataCategory.SiteProjectDoctorate);

            foreach (var innovationData in data)
            {
                result.Add(new ProposingItem()
                               {
                                   ID = innovationData.ID,
                                   Value = innovationData.DataValue,
                               });
            }

            return result;
        }

        private List<SelectListItem> GetAllRoles()
        {
            var serviceClient = new DataGatewayServiceClient();
            var roles = serviceClient.GetAllRoles();

            var result = new List<SelectListItem>();
            foreach (var role in roles)
            {
                result.Add(new SelectListItem() { Text = role.RoleName, Value = role.ID.ToString() });
            }
            return result;
        }

        private IEnumerable<UserModel> GetAllSystemUsers()
        {
            var result = new List<UserModel>();

            var client = new DataGatewayServiceClient();
            var users = client.GetAllSystemUsers();

            foreach (var user in users)
            {
                result.Add(new UserModel()
                               {
                                   UserID = user.ID,
                                   UserName = user.UserName,
                                   FullName = user.FullName,
                                   Email = user.Email,
                                   ContactNumber = user.ContactNumber,
                                   RoleName = user.RoleName,
                                   Status = user.IsDisabled ? "Disabled" : "Enabled",
                                   ChampionFors = BuildProposingItemsDisplayString(user.ChampionForNames)
                               });
            }

            return result;
        }

        private string BuildProposingItemsDisplayString(string[] items)
        {
            string result = string.Empty;
            if (items != null)
            {
                result = string.Join("; ", items);
            }
            return result;
        }

        [HttpGet]
        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator, Constant.Role.InnovationTeam)]
        public ActionResult AddUser()
        {
            ViewBag.Title = "User Management / Add User";

            var model = new UserDetailsModel(UserManagementMode.Add);
            model.Roles = GetAllRoles();
            //model.ADUsers = GetAllADUsers();
            //model.Users = ActiveDirectoryHelper.GetAllUsers();
            model.ProposingItems = GetAllProposingItems();
            model.ChampionFors = GetAllProposingItems();

            return View("UserDetails", model);
        }

        [HttpGet]
        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator, Constant.Role.InnovationTeam)]
        public ActionResult EditUser(int userID)
        {
            ViewBag.Title = "User Management / Edit User";

            var client = new DataGatewayServiceClient();
            var user = client.GetUser(userID);

            if (user != null)
            {
                var model = new UserDetailsModel(UserManagementMode.Edit);
                model.UserID = userID;
                model.UserName = string.IsNullOrEmpty(user.UserName) ? string.Empty : user.UserName.Trim();
                model.FullName = user.FullName;
                model.Email = user.Email;
                model.ContactNumber = user.ContactNumber;
                model.IsDisabled = user.IsDisabled;

                model.RoleID = user.RoleID;
                model.Roles = GetAllRoles();
                model.ProposingItems = GetAllProposingItems();
                model.ChampionFors = GetAllProposingItems();

                foreach (var item in user.ProposingItems)
                {
                    var proposingItem = model.ProposingItems.FirstOrDefault(x => x.ID == item.ID);
                    if (proposingItem != null)
                        proposingItem.IsSelected = true;
                }

                foreach (var item in user.ChampionFors)
                {
                    var championFor = model.ChampionFors.FirstOrDefault(x => x.ID == item.ChampionForID);
                    if (championFor != null)
                        championFor.IsSelected = true;
                }

                return View("UserDetails", model);
            }

            TempData["UserManagement_Error"] = "This user does not exist.";
            return RedirectToAction("UserList");
        }

        [HttpPost]
        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator, Constant.Role.InnovationTeam)]
        public ActionResult SaveUser(UserDetailsModel model)
        {
            var client = new DataGatewayServiceClient();
            var user = new Crossrail.IMS.WebApp.DataGatewayService.User();

            user.RoleID = model.RoleID.Value;
            user.IsDisabled = model.IsDisabled;

            if (model.ProposingItems != null)
                user.ProposingItems = (from item in model.ProposingItems
                                       where item.IsSelected
                                       select new InnovationData()
                                       {
                                           ID = item.ID
                                       }).ToArray();
            if (model.ChampionFors != null)
                user.ChampionFors = (from item in model.ChampionFors
                                     where item.IsSelected
                                     select new UserChampionFor()
                                     {
                                         UserID = model.UserID,
                                         ChampionForID = item.ID
                                     }).ToArray();

            if (model.Mode == UserManagementMode.Add)
            {
                if (!ModelState.IsValid)
                    return AddUser();

                var userName = model.UserName.Trim();
                string error = string.Empty;
                var adUser = ActiveDirectoryHelper.GetUser(userName);
                if (adUser != null)
                {
                    user.UserName = userName;
                    int errorCode = client.AddUser(user, UserNameLog);

                    if (errorCode < 0)
                    {
                        switch (errorCode)
                        {
                            case Constant.UserManagementErrorCode.USER_ALREADY_ADDED:
                                {
                                    error = "This username is already added.";
                                    break;
                                }
                        }
                    }

                }
                else
                {
                    error = "This username does not exist.";
                }

                if (!string.IsNullOrEmpty(error)) //has any errors?
                {
                    ModelState.AddModelError("", error);
                    return AddUser();
                }
            }
            else
            {
                user.ID = model.UserID;
                client.UpdateUser(user, UserNameLog);
            }

            return RedirectToAction("UserList");
        }

        [HttpGet]
        [IMSAuthorizeAttribute(Constant.Role.SystemAdministrator, Constant.Role.InnovationTeam)]
        public ActionResult UserList()
        {
            return View(GetAllSystemUsers());
        }

        public ActionResult CancelAddingUser()
        {
            return RedirectToAction("UserList");
        }

        [HttpGet]
        [AllowAnonymous]
        public ActionResult NotAuthorized()
        {
            return View();
        }

        [AllowAnonymous]
        public ActionResult AccessDenied()
        {
            return View();
        }

        public JsonResult GetUser(string userName)
        {
            var result = ActiveDirectoryHelper.GetUser(userName);
            return Json(result, JsonRequestBehavior.AllowGet);
        }

    }
}
